@@ -64,10 +64,10 @@ html_engine_hide_cursor  (HTMLEngine *engine)
 
 	if ((engine->editable || engine->caret_mode) && engine->cursor_hide_count == 0) {
 		if (!engine->editable) {
-			e = html_object_engine(engine->cursor->object, NULL);
+			e = html_object_engine (engine->cursor->object, NULL);
 			if (e) {
 				e->caret_mode = engine->caret_mode;
-				html_cursor_copy(e->cursor, engine->cursor);
+				html_cursor_copy (e->cursor, engine->cursor);
 			} else	e = engine;
 		}
 		html_engine_draw_cursor_in_area (e, 0, 0, -1, -1);
@@ -79,7 +79,7 @@ html_engine_hide_cursor  (HTMLEngine *engine)
 void
 html_engine_show_cursor  (HTMLEngine *engine)
 {
-        HTMLEngine * e = engine;
+	HTMLEngine * e = engine;
 
 	g_return_if_fail (engine != NULL);
 	g_return_if_fail (HTML_IS_ENGINE (engine));
@@ -89,10 +89,10 @@ html_engine_show_cursor  (HTMLEngine *engine)
 		engine->cursor_hide_count--;
 		if ((engine->editable || engine->caret_mode) && engine->cursor_hide_count == 0) {
 			if (!engine->editable) {
-				e = html_object_engine(engine->cursor->object, NULL);
+				e = html_object_engine (engine->cursor->object, NULL);
 				if (e) {
 					e->caret_mode = engine->caret_mode;
-					html_cursor_copy(e->cursor, engine->cursor);
+					html_cursor_copy (e->cursor, engine->cursor);
 				} else e = engine;
 			}
 			html_engine_draw_cursor_in_area (e, 0, 0, -1, -1);
@@ -119,24 +119,22 @@ draw_cursor_rectangle (HTMLEngine *e, gint x1, gint y1, gint x2, gint y2,
 		       GdkColor *on_color, GdkColor *off_color,
 		       gint offset)
 {
-	GdkGC *gc;
-	GdkColor color;
-	gint8 dashes [2] = { 1, 3 };
+	cairo_t *cr;
+	const double dashes[2] = { 1, 3 };
+	gint ndash = G_N_ELEMENTS (dashes);
 
 	if (x1 > x2 || y1 > y2 || !e->window)
 		return;
 
-	gc = gdk_gc_new (e->window);
-	color = *on_color;
-	gdk_rgb_find_color (gdk_drawable_get_colormap (e->window), &color);
-	gdk_gc_set_foreground (gc, &color);
-	color = *off_color;
-	gdk_rgb_find_color (gdk_drawable_get_colormap (e->window), &color);
-	gdk_gc_set_background (gc, &color);
-	gdk_gc_set_line_attributes (gc, 1, GDK_LINE_DOUBLE_DASH, GDK_CAP_ROUND, GDK_JOIN_ROUND);
-	gdk_gc_set_dashes (gc, offset, dashes, 2);
-	gdk_draw_rectangle (e->window, gc, 0, x1, y1, x2 - x1, y2 - y1);
-	g_object_unref (gc);
+	/* FIXME: what is the off color for? */
+	cr = gdk_cairo_create (e->window);
+	gdk_cairo_set_source_color (cr, on_color);
+	cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
+	cairo_set_dash (cr, dashes, ndash, offset);
+	cairo_rectangle (cr, x1 - 0.5, y1 - 0.5, x2 - x1 + 1, y2 - y1 + 1);
+	cairo_set_line_width (cr, 1.0);
+	cairo_stroke (cr);
+	cairo_destroy (cr);
 }
 
 static gint cursor_enabled = TRUE;
@@ -317,15 +315,15 @@ html_engine_draw_cursor_in_area (HTMLEngine *engine,
 
 	html_object_get_cursor (obj, engine->painter, offset, &x1, &y1, &x2, &y2);
 	while (obj) {
-		if (html_object_is_frame(obj)) {
-			x1 -= HTML_EMBEDDED(obj)->abs_x;
-			x2 -= HTML_EMBEDDED(obj)->abs_x;
-			y1 -= HTML_EMBEDDED(obj)->abs_y;
-			y2 -= HTML_EMBEDDED(obj)->abs_y;
+		if (html_object_is_frame (obj)) {
+			x1 -= HTML_EMBEDDED (obj)->abs_x;
+			x2 -= HTML_EMBEDDED (obj)->abs_x;
+			y1 -= HTML_EMBEDDED (obj)->abs_y;
+			y2 -= HTML_EMBEDDED (obj)->abs_y;
 			break;
 		}
-                obj = obj->parent;
-        }
+		obj = obj->parent;
+	}
 
 	/* get scroll offset */
 	hadj = gtk_layout_get_hadjustment (GTK_LAYOUT (engine->widget));
@@ -341,7 +339,16 @@ html_engine_draw_cursor_in_area (HTMLEngine *engine,
 	gtk_im_context_set_cursor_location (GTK_HTML (engine->widget)->priv->im_context, &pos);
 
 	if (clip_cursor (engine, x, y, width, height, &x1, &y1, &x2, &y2)) {
-		gdk_draw_line (engine->window, engine->invert_gc, x1, y1, x2, y2);
+		cairo_t *cr;
+		cr = gdk_cairo_create (engine->window);
+		cairo_set_source_rgb (cr, 1, 1, 1);
+		cairo_set_operator (cr, CAIRO_OPERATOR_DIFFERENCE);
+		cairo_set_line_cap (cr, CAIRO_LINE_CAP_SQUARE);
+		cairo_move_to (cr, x1 + 0.5, y1 + 0.5);
+		cairo_line_to (cr, x2 + 0.5, y2 - 0.5);
+		cairo_set_line_width (cr, 1);
+		cairo_stroke (cr);
+		cairo_destroy (cr);
 	}
 }
 
